package extractionTools;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang3.text.WordUtils;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

//Class to extract the likely classes for a given animal from several webpage sources.
public class AnimalClassExtractor {
	
	//Get the properties for each animal and return as a Map<string,string> object.
		public static AnimalCounter animalClassExtractor(List<String> animals) throws IOException{

			String result = "";
			AnimalCounter animalCounter = new AnimalCounter();
			List<String> listLinks = Arrays.asList("https://en.wikipedia.org/wiki/", "http://a-z-animals.com/animals/","http://animals.sandiegozoo.org/animals/");

			//For each animal in the list.
			for(String animal : animals){
				System.out.println("\n");
				result = "";
				
				animal = animal.toLowerCase();
				Testing.animalTest(animal, true);

				//For each link in the list of links.
				for(String link : listLinks){
					String URL = link + animal;
					result = beginExtraction(URL);
					
					//Count the number of times that each property comes up
					if (result != ""){
						System.out.println(URL + " classified: " + result);
						animalCounter.addAnimalRecord(animal, result);
					}

					else{
						animalCounter.addAnimalRecord(animal,"NONE");
					}
				}

				//animalCounter.printDictResults(); //animalCounter.mergeResults();
			}
			
			//animalCounter.printDictResults();
			return animalCounter;
		}

	//Function to handle extraction results.
	public static String beginExtraction(String URL){

		try{
			Document extractedHTML = InformationExtractor.getHTML(URL);

			String result = pageAnimalClassExtractor(extractedHTML);
			//Testing.simpleTest(result,true);

			return result;

		//Returns none if no classification information cannot be found.
		}catch(Exception ex){
			ex.printStackTrace();
			return "NONE";
		}

	}

	//Extract the class of an animal given a specific pages HTML.
	public static String pageAnimalClassExtractor(Document extractedHTML){

		String extractionResult = "";
		Elements interimClass = new Elements();

		String infoboxPattern = "table[class = infobox biota]";
		String articleFactsPattern = "table[class = article_facts]";
		String paneContentPattern = "div[class = pane-content]";

		//Check whether these patterns are present in the page.
		Boolean infoboxText      = InformationExtractor.patternExtractor(extractedHTML,infoboxPattern).hasText();
		Boolean articleFactsText = InformationExtractor.patternExtractor(extractedHTML, articleFactsPattern).hasText();
		Boolean paneContentText  = InformationExtractor.patternExtractor(extractedHTML,paneContentPattern).hasText();

		//Get the elements in the document which conform to the specified options.
		//From these elemnts get the "item" string to be returned.
		if (infoboxText == true){

			try{
				String pattern = "table[class = infobox biota] tr:contains(class:) a[title]";
				interimClass = InformationExtractor.patternExtractor(extractedHTML,pattern);
				extractionResult = interimClass.attr("title");
				
				if (extractionResult == ""){
					
					pattern = "table[class = infobox biota] tr:contains(clade:) a[title]";
					interimClass = InformationExtractor.patternExtractor(extractedHTML,pattern);
					extractionResult = interimClass.attr("title");
				}
			}

			catch(Exception ex){
				System.out.println("Parser could not find data : Wikipedia site");
			}

		}

		else if (articleFactsText == true){

			try{
				String pattern = "table[class = article_facts] tr:contains(class:)";
				interimClass = InformationExtractor.patternExtractor(extractedHTML,pattern);

				Element animalClass = (Element) interimClass.select("td").first().nextSibling();
				extractionResult = animalClass.text();
			}

			catch(Exception ex){
				System.out.println("Parser could not find data : articleFacts site");
				return "NONE";
			}
		}

		else if (paneContentText == true){

			try{
				String pattern = "div[class=pane-content] div[class=double-field-second]";
				interimClass = InformationExtractor.patternExtractor(extractedHTML, pattern);

				Element animalClass= (Element) interimClass.select("div").first();
				extractionResult = animalClass.text();
			}

			catch(Exception ex){
				System.out.println("Parser could not find data : paneContent site");
			}
			
		}
		return extractionResult;
	}

}


