package extractionTools;

import java.util.Arrays;
import java.util.List;
import java.util.TreeMap;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

//Class to compare how similar an animal is to the expected animalClasses.
public class AnimalClassSimilarity {

	//Main function to return an animalsClassification.
	public static String animalClassify(String animalClass){

		//System.out.println("Checking:" + animalClass);
		List<String> validAnimalClasses= Arrays.asList("Reptile", "Mammal","Amphibian","Bird","Fish","Insect","Dinosaur");
		double largestPercentageSimilarity = 0;
		String probableAnimal = "NONE";

		//For each animal in the list.
		for(String validAnimal : validAnimalClasses){
			//Get the percentage of how similar the classes are.
			double percentageSimilarity = checkClassification(animalClass,validAnimal);

			if(percentageSimilarity >= largestPercentageSimilarity){

				//The most similar percentage will return the most similar animal.
				largestPercentageSimilarity = percentageSimilarity;
				probableAnimal = validAnimal;
			}
		}

		//If the largestPercentage is less than 50 . 
		//I.e the animal is less than half as similar as one of the animal classes.
		if (largestPercentageSimilarity < 50){
			//System.out.println(animalClass + " : Most similar to a " + probableAnimal);
			//System.out.println("Checking other cases which may be relevant");
			probableAnimal= checkSpecialCases(animalClass);
		}

		return probableAnimal;
	}

	//Use levenshtein distance to compare two animal strings.
	public static double checkClassification(String animal1,String animal2){

		//Convert to lowercase to prevent problems when analysing strange cases.
		animal1 = animal1.toLowerCase();
		animal2 = animal2.toLowerCase();
		
		//Work out the percentage the animals differ.
		int overallLength = (animal1.length() + animal2.length())/2;
		int distance= StringUtils.getLevenshteinDistance(animal1,animal2);
		double percentage  = (double) distance / overallLength * 100;

		//Return the percentage which the animals are similar.
		return 100 - percentage;
	}

	//Check other cases such as Aves -> Bird
	public static String checkSpecialCases(String animal){
		String returnAnimal = "NONE";
		Map<String,String> acceptableAnimalNames = new TreeMap<String,String>(String.CASE_INSENSITIVE_ORDER);

		acceptableAnimalNames.put("Bird", "aves");
		acceptableAnimalNames.put("Fish", "Actinopterygii,Chondrichthyes,Crustacea,Malacostraca");
		acceptableAnimalNames.put("Mammal", "Mammalia");

		//For each entry in the acceptableAnimalNames dictionary 
		for(Map.Entry<String, String> animalEntry : acceptableAnimalNames.entrySet()){

			//Get the possible animals from the "values" part of the dictionary.
			for( String possibleAnimal : animalEntry.getValue().split(",") ){

				//Check if any of the specialcases fit the specified animal.
				if (animal.replaceAll("\\s\\(.*\\)", "").equalsIgnoreCase(possibleAnimal)){
					//System.out.println(animal + possibleAnimal);
					returnAnimal = animalEntry.getKey();

				}		
			}
		}

		return returnAnimal;
	}

	//	public static void main(String[] args){
	//	
	//	List<String> validAnimalClasses= Arrays.asList("Reptilia", "Mammalia(Mammals)","Amphibia","Aves","Actinopterygii");
	//	
	////	for(String validAnimal : validAnimalClasses){
	////		System.out.println(validAnimal + "->" + animalClassify(validAnimal));
	////	
	////	}
	//	
	//	System.out.println("Mammalia(Mammals)" + "->" + animalClassify("Mammalia (Mammals)"));
	//	
	//	//System.out.println("Mammalia".equalsIgnoreCase("Mammalia"));
	//
	//}


}
