package extractionTools;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.Set;

//Data structure object used to count the number of occurrences of a particular property.
//e.g Lion = [Mammal,Mammal,Reptile] will count the number of times Mammal occurs.
public class AnimalCounter {

	//Globally declared dictionary for the data structure.
	private Map<String,String> animalDict = new HashMap<String,String>();

	//Function to add a record based on an animal name and an associated property.
	public void addAnimalRecord(String animal,String property){

		//Append to the dictionary the property if the key is already stored.
		if(animalDict.containsKey(animal)){
			animalDict.put(animal, animalDict.get(animal) + "," + property);
		}
		//Or create a new key value pair and add this if the key does not already exist.
		else{
			animalDict.put(animal, property);
		}	
	}

	//Function to print the results stored in the dictionary.
	public void printDictResults(){

		//For each entry ,print the associated key and value for that entry.
		for ( Map.Entry<String, String> entry : animalDict.entrySet() ) {
			System.out.println(entry.getKey() + ":" + entry.getValue());
		}
	}

	//Function to merge together the results contained inside of the dictionary.
	public Map<String, String> mergeResults() throws IOException{

		//Set to none just in case no classification can be found for that animal.
		String valueToBeAdded = "NONE";

		//Create a set firstly and convert this to an ArrayList to prevent Concurrency errors.
		Set<String> keySet = animalDict.keySet();
		
		List<String> keyList = new ArrayList<String>(keySet);

		Iterator<String> keyIterator = keyList.iterator();

				//For each entry in the animalDictionary.
				while ( keyIterator.hasNext() == true ) {
					
						String animalKey = keyIterator.next();
						System.out.println(animalKey);
						
						String[] values = animalDict.get(animalKey).split(",");
						
						//Declare the valuesDict to calculate how often each class occurs.
						Map<String,Integer> valuesDict = new HashMap<String,Integer>();

						//For each of the values in the values array.
						//Check each value and whether it can be mapped to an accepted name and equal valueToBeAdded.
						//Reptilia -> Reptile and Aves-> Bird as examples.
						for(String value : values){
							valueToBeAdded = AnimalClassSimilarity.animalClassify(value);

							//If that key has already been added to the valuesDict
							//Increment the value counter by 1.
							if(valuesDict.containsKey(valueToBeAdded)){
								valuesDict.put(valueToBeAdded,  valuesDict.get(valueToBeAdded) + 1);
							}

							//Else add that key and and a count of 1.
							else{
								valuesDict.put(valueToBeAdded, 1); 
							}
						}

						//Declarations
						int largestCount = 0;
						String largestKeyCount = "NONE";

						//For each numberResult in the valuesDict set of entries.
						for(Map.Entry<String, Integer> numberResult : valuesDict.entrySet() ){

							//Only if the count stored is greater than the largest number
							if(numberResult.getValue() >= largestCount){

								//Do nothing if the key is NONE,this means no classification data was gathered.
								if(numberResult.getKey().equals("NONE")){ 
								}

								//Else update the largestKey and associated count values to the ones stored in that entry.
								else{
									largestKeyCount = numberResult.getKey();
									largestCount = numberResult.getValue();
								}

							}
							System.out.println("\n");

						}

						System.out.println(largestKeyCount);

						Testing.animalTest(animalKey, true);
						
						largestKeyCount = manualInput("\nDo you know the classification(Y/N)?", animalKey, largestKeyCount);
						
						System.out.println(animalKey + "-" + "Class to be added:" + largestKeyCount );

						animalDict.remove(animalKey);

						String propertiesToBeAdded = PropertyExtractor.propertyFind(animalKey);
						
						if(propertiesToBeAdded.equals("NOPROPERTIESFOUND")){
							propertiesToBeAdded = manualInput("\nDo you know the properties(Y/N)", animalKey, propertiesToBeAdded);
						}

						//Add the key of the entry,animal classification and properties associated with that key.
						animalDict.put(animalKey + "#" + largestKeyCount, propertiesToBeAdded);
					} 


		//Returns the dictionary which is now of the form.
		// {Ostrich, [Bird#wings]} - the animal name,followed by a set of values of the animals classifcation and properties.

		System.out.println("\n Animal Dictionary contains \n" +  animalDict);

		return animalDict;
	}

	public static Map<String,String> addAdditionalPropertiesToDict(Map<String,String> animalDict){

		//List<String> animalClasses = Arrays.asList("Mammals","Reptiles","Amphibians","Fishes","Birds");
		//String classlink = "http://kids.sandiegozoo.org/animals/";

		Map<String, String> animalClasses = new HashMap<String,String>();
		animalClasses.put("Bird#ANIMALCLASS", "winged");
		animalClasses.put("Reptile#ANIMALCLASS", "scaled");
		animalClasses.put("Snake#Reptile", "scaled");
		animalClasses.put("Fish#ANIMALCLASS","gills");
		animalClasses.put("Mammal#ANIMALCLASS","hairy");
		animalClasses.put("Amphibian#ANIMALCLASS", "coldBlooded,eggs");
		animalClasses.put("poison_frog", "eggs,coldBlooded");
		
		animalClasses.put("Unclassified#ANIMALCLASS","NOPROPERTIESFOUND");

		animalDict.putAll(animalClasses);

		return animalDict;	
	}

	//Retrieve manual input from the user
	public static String manualInput(String question,String animal,String noneResult){
		String returnedVal = noneResult;
		
		//Check whether the properties or class requires manual input.
		if(noneResult.equals("NONE") || noneResult.equals("NOPROPERTIESFOUND")){

			System.out.println(animal + " cannot be classified by system");

			System.out.println(question);

			Scanner scanner = new Scanner(System.in);
			
			//Check whether user wants to provide input.
			String response = scanner.nextLine();

			switch(response.toLowerCase()){
			case "y" :
				
				//Check whether the class needs to be added.
				if (noneResult.equals("NONE")){
					returnedVal= manualClassInput(scanner);
					break;
				}
				
				//Check whether the property needs to be added.
				else if (noneResult.equals("NOPROPERTIESFOUND")){
					returnedVal = manualPropertyInput(scanner);
					break;
				}
						
			default :
				System.out.println("Animal will not be classified");
				break;
			}

		}
		return returnedVal;
	}

	//Get manual input for the animal's classification.
	public static String manualClassInput(Scanner scanner){
		
		List<String> acceptableAnimalClassesArray = Arrays.asList("mammal","reptile","amphibian","fish","bird");
		Set<String> acceptableAnimalClasses = new HashSet<String>(acceptableAnimalClassesArray);

		System.out.println("The acceptable classes currently supported are:" + acceptableAnimalClasses);
		System.out.println("Any other input will not be added\n");

		System.out.println("Please input your classification:");

		//Retrieve the class to be added and convert to lowercase.
		String animalClass = scanner.nextLine();
		
	    animalClass = AnimalClassSimilarity.animalClassify(animalClass);
		
		animalClass = animalClass.toLowerCase();
		

		//If the class is acceptable add it.
		if(acceptableAnimalClasses.contains(animalClass.toLowerCase())){
			System.out.println("Thanks " + animalClass + " has been added");

			//Return the result capitalised.
			return animalClass.substring(0, 1).toUpperCase() + animalClass.substring(1);
		}

		//If no valid input.
		else{
			System.out.println("Sorry could not be added " + animalClass + " is not a currently accepted class");
			System.out.println("The acceptable classes are:" + acceptableAnimalClasses);
			return "NONE";
		}

	}
	
	//Get manual input for the animal's properties.
	public static String manualPropertyInput(Scanner scanner){
		
		//List should be lowercase.
		List<String> acceptableAnimalPropertiesArray = Arrays.asList("scales","coldblooded","gills","vertebrate","winged","hairy");
		Set<String> acceptableAnimalProperties = new HashSet<String>(acceptableAnimalPropertiesArray);
		Boolean mistakeMade = false;

		System.out.println("The acceptable properties currently supported are:" + acceptableAnimalProperties);
		System.out.println("Any other input will not be accepted\n");

		System.out.println("Please input the animal's properties(seperated by commas):");

		//Retrieve the manual input of the properties.
		String animalProperties = scanner.nextLine();
		animalProperties = animalProperties.toLowerCase();
		
		//Check that each of the properties is valid.
		for(String property: animalProperties.toLowerCase().split(",")){
			
			System.out.println("Trying:" + property);
			if(acceptableAnimalProperties.contains(property.toLowerCase())){
				System.out.println("Thanks " + property + " has been added");
				
			}
			
			else{
				System.out.println("Error:Sorry could not be added " + property + " is not a currently accepted property");
				System.out.println("Please note that properties MUST be seperated by commas.");
				
				System.out.println("The acceptable properties are:" + acceptableAnimalProperties);
				
				mistakeMade = true;
			}

		}
		
		if(mistakeMade == true){
				return "NOPROPERTIES";
		}
		
		else{
			return animalProperties;
		}
		
	}

}

