package extractionTools;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.jsoup.nodes.Document;
import org.jsoup.select.Elements;

import com.hp.hpl.jena.ontology.ObjectProperty;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.Ontology;

public class FoodChain {
	
	static int count = 0;

	//Recurse through the tree of animal's "eaten" and "eatenBy" the recursed animal.
	public static void animalRecurse(String animal,List<String> acceptedAnimals,ObjectProperty eats,ObjectProperty eatenBy, 
			OntologyCreator oCreator,LinkedList<String> eatsArray,LinkedList<String> eatenByArray,int recursionCount,int maxRecursionCount ){

		Document extractedHTML = new Document("");
		Elements preyHTML = new Elements();
		Elements predHTML = new Elements();
		
		//Convert plurals to singulars. Lions -> Lion
		if(acceptedAnimals.contains(animal.replaceAll("s$","").toLowerCase())){
			animal= animal.replaceAll("s$","").toLowerCase();
		}
		
		else if(acceptedAnimals.contains(animal.replaceAll("es$",""))){
			animal = animal.replaceAll("es$","").toLowerCase();
		}
		
		OntClass animalClass = oCreator.createOntClass(animal);

		System.out.println("The animal being examined is:" + animal);
		System.out.println("Current eats:" +  eatsArray);
		System.out.println("Current eatenBy:" +  eatenByArray);
		System.out.println("\n");
		System.out.println("Recursion count:" + recursionCount);

		final String link = "http://a-z-animals.com/animals/";

		//This is false when a value has a space
		//wild boar should be mapped to wild_boar
		if (Arrays.asList(animal.split(" ")).contains(animal) == false){
			//Use the join method to add in the underscore
			animal = String.join("_", animal.split(" "));
		}

		try {

			extractedHTML = InformationExtractor.getHTML(link + animal);

			preyHTML = InformationExtractor.patternExtractor(extractedHTML, "table[class = article_facts] tr:contains(prey:) td");
			predHTML = InformationExtractor.patternExtractor(extractedHTML, "table[class = article_facts] tr:contains(predators:) td");

		} catch (Exception e) {
			System.out.println("Extraction error: Predator or prey information could not be extracted");
		}

		//Checking for prey recursions.
		try{
			//If prey were extracted
			if (preyHTML.hasText() == true){

				//Get the prey animal's names.
				String preyString = preyHTML.get(1).text();

				//Split if theres several prey.
				String[] preyList = preyString.split(", ");

				//Print the prey as a list.
				System.out.println("Prey:" + Arrays.toString(preyList));

				//For each of the prey in the list.
				for(String prey : preyList){
					
					//Convert plurals to singulars. Lions -> Lion
					
					if(acceptedAnimals.contains(prey.replaceAll("s$","").toLowerCase())){
						prey = prey.replaceAll("s$","").toLowerCase();
						
					}
					
					else if(acceptedAnimals.contains(prey.replaceAll("es$","").toLowerCase())){
						prey = prey.replaceAll("es$","").toLowerCase();
					}

					System.out.println("Prey being examined:" + prey + "\n");

					String proposedAnimalRelation = animal + "#eats:" + prey;

					//Perform recursive checks.
					if(acceptedAnimals.contains(prey.toLowerCase()) && (infiniteLoopCheck(eatsArray, eatenByArray, proposedAnimalRelation) == false)
							&& (recursionCount <= maxRecursionCount)){

						//Add the prey name to the various objects where it is needed.
						eatsArray.add(animal + "#" + prey);
						OntClass preyClass = oCreator.createOntClass(prey);
						System.out.println("Adding prey Triple relation");
						oCreator.addTriple(animalClass,eats,preyClass);
						count++;

						//Recurse through the prey animal name.
						animalRecurse(prey, acceptedAnimals,eats,eatenBy,oCreator,eatsArray,eatenByArray,recursionCount+=1,maxRecursionCount);

					}

				}
			}

			//If predators were extracted
			if (predHTML.hasText() == true){
				
				//Get the predators names.
				String predString = predHTML.get(1).text();
				
				//Split if theres several predators.
				String[] predList= predString.split(", ");

				System.out.println("Predators:" + Arrays.toString(predList));

				//For each of the predators in the list.
				for(String pred : predList){
					
					System.out.println(pred.replaceAll("s$","") );

					//Convert plurals to singulars. Lions -> Lion
					if(acceptedAnimals.contains(pred.replaceAll("s$","").toLowerCase())){
						pred = pred.replaceAll("s$","").toLowerCase();
					}
					
					else if(acceptedAnimals.contains(pred.replaceAll("es$","").toLowerCase())){
						pred = pred.replaceAll("es$","").toLowerCase();
					}
					
					System.out.println("Prey being examined:" + pred);
					String proposedAnimalRelation = animal + "#eatenBy:" + pred;

					//Perform recursive checks.
					if(acceptedAnimals.contains(pred.toLowerCase()) && (infiniteLoopCheck(eatsArray, eatenByArray, proposedAnimalRelation) == false)
							&& (recursionCount <= maxRecursionCount)){
						
						//Add the predators name to the various objects where it is needed.
						eatenByArray.add(animal + "#" + pred);
						OntClass predClass = oCreator.createOntClass(pred);

						System.out.println("Adding predator relation");
						oCreator.addTriple(animalClass,eatenBy,predClass);
						count++;

						//Recurse through the predator animal name.
						animalRecurse(pred,acceptedAnimals,eats,eatenBy,oCreator,eatsArray,eatenByArray,recursionCount+=1,maxRecursionCount);
					}

				}
			}

		}

		catch(Exception ex){
			ex.printStackTrace();
		}
	}

	public static OntModel createFoodChain(String startAnimal, OntologyCreator oCreator){

		final String baseURI = "http://www.example.com/seanjames/ontologies/animals/";
		LinkedList<String> eatsArray = new LinkedList<String>();
		LinkedList<String> eatenByArray = new LinkedList<String>();
		List<String> acceptedAnimals = InformationExtractor.animalListFinder("http://a-z-animals.com/animals/");

		//	List<String> acceptedAnimals = Arrays.asList(abyssinian, adelie penguin, affenpinscher, afghan hound, african bush elephant, african civet, african clawed frog, african forest elephant, african palm civet, african penguin, african tree toad, african wild dog, ainu dog, airedale terrier, akbash, akita, alaskan malamute, albatross, aldabra giant tortoise, alligator, alpine dachsbracke, american bulldog, american cocker spaniel, american coonhound, american eskimo dog, american foxhound, american pit bull terrier, american staffordshire terrier, american water spaniel, anatolian shepherd dog, angelfish, ant, anteater, antelope, appenzeller dog, arctic fox, arctic hare, arctic wolf, armadillo, asian elephant, asian giant hornet, asian palm civet, asiatic black bear, australian cattle dog, australian kelpie dog, australian mist, australian shepherd, australian terrier, avocet, axolotl, aye aye, baboon, bactrian camel, badger, balinese, banded palm civet, bandicoot, barb, barn owl, barnacle, barracuda, basenji dog, basking shark, basset hound, bat, bavarian mountain hound, beagle, bear, bearded collie, bearded dragon, beaver, bedlington terrier, beetle, bengal tiger, bernese mountain dog, bichon frise, binturong, bird, birds of paradise, birman, bison, black bear, black rhinoceros, black russian terrier, black widow spider, bloodhound, blue lacy dog, blue whale, bluetick coonhound, bobcat, bolognese dog, bombay, bongo, bonobo, booby, border collie, border terrier, bornean orang-utan, borneo elephant, boston terrier, bottle nosed dolphin, boxer dog, boykin spaniel, brazilian terrier, brown bear, budgerigar, buffalo, bull mastiff, bull shark, bull terrier, bulldog, bullfrog, bumble bee, burmese, burrowing frog, butterfly, butterfly fish, caiman, caiman lizard, cairn terrier, camel, canaan dog, capybara, caracal, carolina dog, cassowary, cat, caterpillar, catfish, cavalier king charles spaniel, centipede, cesky fousek, chameleon, chamois, cheetah, chesapeake bay retriever, chicken, chihuahua, chimpanzee, chinchilla, chinese crested dog, chinook, chinstrap penguin, chipmunk, chow chow, cichlid, clouded leopard, clown fish, clumber spaniel, coati, cockroach, collared peccary, collie, common buzzard, common frog, common loon, common toad, coral, cottontop tamarin, cougar, cow, coyote, crab, crab-eating macaque, crane, crested penguin, crocodile, cross river gorilla, curly coated retriever, cuscus, cuttlefish, dachshund, dalmatian, darwin's frog, deer, desert tortoise, deutsche bracke, dhole, dingo, discus, doberman pinscher, dodo, dog, dogo argentino, dogue de bordeaux, dolphin, donkey, dormouse, dragonfly, drever, duck, dugong, dunker, dusky dolphin, dwarf crocodile, eagle, earwig, eastern gorilla, eastern lowland gorilla, echidna, edible frog, egyptian mau, electric eel, elephant, elephant seal, elephant shrew, emperor penguin, emperor tamarin, emu, english cocker spaniel, english shepherd, english springer spaniel, entlebucher mountain dog, epagneul pont audemer, eskimo dog, estrela mountain dog, falcon, fennec fox, ferret, field spaniel, fin whale, finnish spitz, fire-bellied toad, fish, fishing cat, flamingo, flat coat retriever, flounder, fly, flying squirrel, fossa, fox, fox terrier, french bulldog, frigatebird, frilled lizard, frog, fur seal, galapagos penguin, galapagos tortoise, gar, gecko, gentoo penguin, geoffroys tamarin, gerbil, german pinscher, german shepherd, gharial, giant african land snail, giant clam, giant panda bear, giant schnauzer, gibbon, gila monster, giraffe, glass lizard, glow worm, goat, golden lion tamarin, golden oriole, golden retriever, goose, gopher, gorilla, grasshopper, great dane, great white shark, greater swiss mountain dog, green bee-eater, greenland dog, grey mouse lemur, grey reef shark, grey seal, greyhound, grizzly bear, grouse, guinea fowl, guinea pig, guppy, hammerhead shark, hamster, hare, harrier, havanese, hedgehog, hercules beetle, hermit crab, heron, highland cattle, himalayan, hippopotamus, honey bee, horn shark, horned frog, horse, horseshoe crab, howler monkey, human, humboldt penguin, hummingbird, humpback whale, hyena, ibis, ibizan hound, iguana, impala, indian elephant, indian palm squirrel, indian rhinoceros, indian star tortoise, indochinese tiger, indri, insect, irish setter, irish wolfhound, jack russel, jackal, jaguar, japanese chin, japanese macaque, javan rhinoceros, javanese, jellyfish, kakapo, kangaroo, keel billed toucan, killer whale, king crab, king penguin, kingfisher, kiwi, koala, komodo dragon, kudu, labradoodle, labrador retriever, ladybird, leaf-tailed gecko, lemming, lemur, leopard, leopard cat, leopard seal, leopard tortoise, liger, lion, lionfish, little penguin, lizard, llama, lobster, long-eared owl, lynx, macaroni penguin, macaw, magellanic penguin, magpie, maine coon, malayan civet, malayan tiger, maltese, manatee, mandrill, manta ray, marine toad, markhor, marsh frog, masked palm civet, mastiff, mayfly, meerkat, millipede, minke whale, mole, molly, mongoose, mongrel, monitor lizard, monkey, monte iberia eleuth, moorhen, moose, moray eel, moth, mountain gorilla, mountain lion, mouse, mule, neanderthal, neapolitan mastiff, newfoundland, newt, nightingale, norfolk terrier, norwegian forest, numbat, nurse shark, ocelot, octopus, okapi, old english sheepdog, olm, opossum, orang-utan, ostrich, otter, oyster, pademelon, panther, parrot, patas monkey, peacock, pekingese, pelican, penguin, persian, pheasant, pied tamarin, pig, pika, pike, pink fairy armadillo, piranha, platypus, pointer, poison dart frog, polar bear, pond skater, poodle, pool frog, porcupine, possum, prawn, proboscis monkey, puffer fish, puffin, pug, puma, purple emperor, puss moth, pygmy hippopotamus, pygmy marmoset, quail, quetzal, quokka, quoll, rabbit, raccoon, raccoon dog, radiated tortoise, ragdoll, rat, rattlesnake, red knee tarantula, red panda, red wolf, red-handed tamarin, reindeer, rhinoceros, river dolphin, river turtle, robin, rock hyrax, rockhopper penguin, roseate spoonbill, rottweiler, royal penguin, russian blue, sabre-toothed tiger, saint bernard, salamander, sand lizard, saola, scorpion, scorpion fish, sea dragon, sea lion, sea otter, sea slug, sea squirt, sea turtle, sea urchin, seahorse, seal, serval, sheep, shih tzu, shrimp, siamese, siamese fighting fish, siberian, siberian husky, siberian tiger, silver dollar, skunk, sloth, slow worm, snail, snake, snapping turtle, snowshoe, snowy owl, somali, south china tiger, spadefoot toad, sparrow, spectacled bear, sperm whale, spider monkey, spiny dogfish, sponge, squid, squirrel, squirrel monkey, sri lankan elephant, staffordshire bull terrier, stag beetle, starfish, stellers sea cow, stick insect, stingray, stoat, striped rocket frog, sumatran elephant, sumatran orang-utan, sumatran rhinoceros, sumatran tiger, sun bear, swan, tang, tapir, tarsier, tasmanian devil, tawny owl, termite, tetra, thorny devil, tibetan mastiff, tiffany, tiger, tiger salamander, tiger shark, tortoise, toucan, tree frog, tropicbird, tuatara, turkey, turkish angora, uakari, uguisu, umbrellabird, vampire bat, vervet monkey, vulture, wallaby, walrus, warthog, wasp, water buffalo, water dragon, water vole, weasel, welsh corgi, west highland terrier, western gorilla, western lowland gorilla, whale shark, whippet, white faced capuchin, white rhinoceros, white tiger, wild boar, wildebeest, wolf, wolverine, wombat, woodlouse, woodpecker, woolly mammoth, woolly monkey, wrasse, x-ray tetra, yak, yellow-eyed penguin, yorkshire terrier, zebra, zebra shark, zebu, zonkey, zorse, Turtles, Mammals]

		//		acceptedAnimals.add("Turtles");
		//		acceptedAnimals.add("Mammals");
		acceptedAnimals.add("Human");
		System.out.println(acceptedAnimals);

		OntModel animalModel = oCreator.getModel();

		//ObjectProperty eats = animalModel.createTransitiveProperty(baseURI + "eats");
		ObjectProperty eats = animalModel.createObjectProperty(baseURI + "eats");
		ObjectProperty eatenBy = animalModel.createObjectProperty(baseURI + "eatenBy");
		eats.addInverseOf(eatenBy);

		//while(animalsSoFar.size() <= 5){

		try{
			Testing.printTest(startAnimal, true);
			//				//animalRecurse(startAnimal, acceptedAnimals,eats,oCreator,recurseChecker);
			//				
			//				animalsAddedSoFar = new LinkedList<String>();

			int recursionCount = 0;
			
			animalRecurse(startAnimal,acceptedAnimals,eats, eatenBy, oCreator,eatsArray,eatenByArray,recursionCount,10000);
			
			System.out.println("No of recursions:" + count);
			
		}

		catch(Exception ex){

			System.out.println("Problem in recursion of animal:" + startAnimal);
			ex.printStackTrace();
		}

		animalModel = oCreator.getModel();

		return animalModel;
	}

	public static Boolean infiniteLoopCheck(LinkedList<String> eatsArray,LinkedList<String> eatenByArray,String proposedAnimalRelation){

		Boolean infiniteLoopCheck = false;

		System.out.println("Eats:" + eatsArray);
		System.out.println("Eaten By:" + eatenByArray);

		String[] animalRelation = proposedAnimalRelation.split(":");

		String[] animalAndRelation = animalRelation[0].split("#");

		String rebuiltRelation = animalAndRelation[0] + "#" + animalRelation[1];

		if(animalAndRelation[1].equals("eats")){

			if(eatsArray.contains(rebuiltRelation)){
				infiniteLoopCheck=true;
			}
		}

		else if(animalAndRelation[1].equals("eatenBy")){

			if(eatenByArray.contains(rebuiltRelation)){
				infiniteLoopCheck=true;
			}
		}


		else{
			infiniteLoopCheck=false;
		}

		return infiniteLoopCheck;

	}
}




//	public static void main(String[] args){
//
//		final String baseURI = "http://www.example.com/seanjames/ontologies/animals/";
//		LinkedList<String> animalsAddedSoFar = new LinkedList<String>();
//		
//		String startAnimal = "crocodile";
//		List<String> acceptedAnimals = InformationExtractor.animalListFinder("http://a-z-animals.com/animals/");
//		
//	//	List<String> acceptedAnimals = Arrays.asList(abyssinian, adelie penguin, affenpinscher, afghan hound, african bush elephant, african civet, african clawed frog, african forest elephant, african palm civet, african penguin, african tree toad, african wild dog, ainu dog, airedale terrier, akbash, akita, alaskan malamute, albatross, aldabra giant tortoise, alligator, alpine dachsbracke, american bulldog, american cocker spaniel, american coonhound, american eskimo dog, american foxhound, american pit bull terrier, american staffordshire terrier, american water spaniel, anatolian shepherd dog, angelfish, ant, anteater, antelope, appenzeller dog, arctic fox, arctic hare, arctic wolf, armadillo, asian elephant, asian giant hornet, asian palm civet, asiatic black bear, australian cattle dog, australian kelpie dog, australian mist, australian shepherd, australian terrier, avocet, axolotl, aye aye, baboon, bactrian camel, badger, balinese, banded palm civet, bandicoot, barb, barn owl, barnacle, barracuda, basenji dog, basking shark, basset hound, bat, bavarian mountain hound, beagle, bear, bearded collie, bearded dragon, beaver, bedlington terrier, beetle, bengal tiger, bernese mountain dog, bichon frise, binturong, bird, birds of paradise, birman, bison, black bear, black rhinoceros, black russian terrier, black widow spider, bloodhound, blue lacy dog, blue whale, bluetick coonhound, bobcat, bolognese dog, bombay, bongo, bonobo, booby, border collie, border terrier, bornean orang-utan, borneo elephant, boston terrier, bottle nosed dolphin, boxer dog, boykin spaniel, brazilian terrier, brown bear, budgerigar, buffalo, bull mastiff, bull shark, bull terrier, bulldog, bullfrog, bumble bee, burmese, burrowing frog, butterfly, butterfly fish, caiman, caiman lizard, cairn terrier, camel, canaan dog, capybara, caracal, carolina dog, cassowary, cat, caterpillar, catfish, cavalier king charles spaniel, centipede, cesky fousek, chameleon, chamois, cheetah, chesapeake bay retriever, chicken, chihuahua, chimpanzee, chinchilla, chinese crested dog, chinook, chinstrap penguin, chipmunk, chow chow, cichlid, clouded leopard, clown fish, clumber spaniel, coati, cockroach, collared peccary, collie, common buzzard, common frog, common loon, common toad, coral, cottontop tamarin, cougar, cow, coyote, crab, crab-eating macaque, crane, crested penguin, crocodile, cross river gorilla, curly coated retriever, cuscus, cuttlefish, dachshund, dalmatian, darwin's frog, deer, desert tortoise, deutsche bracke, dhole, dingo, discus, doberman pinscher, dodo, dog, dogo argentino, dogue de bordeaux, dolphin, donkey, dormouse, dragonfly, drever, duck, dugong, dunker, dusky dolphin, dwarf crocodile, eagle, earwig, eastern gorilla, eastern lowland gorilla, echidna, edible frog, egyptian mau, electric eel, elephant, elephant seal, elephant shrew, emperor penguin, emperor tamarin, emu, english cocker spaniel, english shepherd, english springer spaniel, entlebucher mountain dog, epagneul pont audemer, eskimo dog, estrela mountain dog, falcon, fennec fox, ferret, field spaniel, fin whale, finnish spitz, fire-bellied toad, fish, fishing cat, flamingo, flat coat retriever, flounder, fly, flying squirrel, fossa, fox, fox terrier, french bulldog, frigatebird, frilled lizard, frog, fur seal, galapagos penguin, galapagos tortoise, gar, gecko, gentoo penguin, geoffroys tamarin, gerbil, german pinscher, german shepherd, gharial, giant african land snail, giant clam, giant panda bear, giant schnauzer, gibbon, gila monster, giraffe, glass lizard, glow worm, goat, golden lion tamarin, golden oriole, golden retriever, goose, gopher, gorilla, grasshopper, great dane, great white shark, greater swiss mountain dog, green bee-eater, greenland dog, grey mouse lemur, grey reef shark, grey seal, greyhound, grizzly bear, grouse, guinea fowl, guinea pig, guppy, hammerhead shark, hamster, hare, harrier, havanese, hedgehog, hercules beetle, hermit crab, heron, highland cattle, himalayan, hippopotamus, honey bee, horn shark, horned frog, horse, horseshoe crab, howler monkey, human, humboldt penguin, hummingbird, humpback whale, hyena, ibis, ibizan hound, iguana, impala, indian elephant, indian palm squirrel, indian rhinoceros, indian star tortoise, indochinese tiger, indri, insect, irish setter, irish wolfhound, jack russel, jackal, jaguar, japanese chin, japanese macaque, javan rhinoceros, javanese, jellyfish, kakapo, kangaroo, keel billed toucan, killer whale, king crab, king penguin, kingfisher, kiwi, koala, komodo dragon, kudu, labradoodle, labrador retriever, ladybird, leaf-tailed gecko, lemming, lemur, leopard, leopard cat, leopard seal, leopard tortoise, liger, lion, lionfish, little penguin, lizard, llama, lobster, long-eared owl, lynx, macaroni penguin, macaw, magellanic penguin, magpie, maine coon, malayan civet, malayan tiger, maltese, manatee, mandrill, manta ray, marine toad, markhor, marsh frog, masked palm civet, mastiff, mayfly, meerkat, millipede, minke whale, mole, molly, mongoose, mongrel, monitor lizard, monkey, monte iberia eleuth, moorhen, moose, moray eel, moth, mountain gorilla, mountain lion, mouse, mule, neanderthal, neapolitan mastiff, newfoundland, newt, nightingale, norfolk terrier, norwegian forest, numbat, nurse shark, ocelot, octopus, okapi, old english sheepdog, olm, opossum, orang-utan, ostrich, otter, oyster, pademelon, panther, parrot, patas monkey, peacock, pekingese, pelican, penguin, persian, pheasant, pied tamarin, pig, pika, pike, pink fairy armadillo, piranha, platypus, pointer, poison dart frog, polar bear, pond skater, poodle, pool frog, porcupine, possum, prawn, proboscis monkey, puffer fish, puffin, pug, puma, purple emperor, puss moth, pygmy hippopotamus, pygmy marmoset, quail, quetzal, quokka, quoll, rabbit, raccoon, raccoon dog, radiated tortoise, ragdoll, rat, rattlesnake, red knee tarantula, red panda, red wolf, red-handed tamarin, reindeer, rhinoceros, river dolphin, river turtle, robin, rock hyrax, rockhopper penguin, roseate spoonbill, rottweiler, royal penguin, russian blue, sabre-toothed tiger, saint bernard, salamander, sand lizard, saola, scorpion, scorpion fish, sea dragon, sea lion, sea otter, sea slug, sea squirt, sea turtle, sea urchin, seahorse, seal, serval, sheep, shih tzu, shrimp, siamese, siamese fighting fish, siberian, siberian husky, siberian tiger, silver dollar, skunk, sloth, slow worm, snail, snake, snapping turtle, snowshoe, snowy owl, somali, south china tiger, spadefoot toad, sparrow, spectacled bear, sperm whale, spider monkey, spiny dogfish, sponge, squid, squirrel, squirrel monkey, sri lankan elephant, staffordshire bull terrier, stag beetle, starfish, stellers sea cow, stick insect, stingray, stoat, striped rocket frog, sumatran elephant, sumatran orang-utan, sumatran rhinoceros, sumatran tiger, sun bear, swan, tang, tapir, tarsier, tasmanian devil, tawny owl, termite, tetra, thorny devil, tibetan mastiff, tiffany, tiger, tiger salamander, tiger shark, tortoise, toucan, tree frog, tropicbird, tuatara, turkey, turkish angora, uakari, uguisu, umbrellabird, vampire bat, vervet monkey, vulture, wallaby, walrus, warthog, wasp, water buffalo, water dragon, water vole, weasel, welsh corgi, west highland terrier, western gorilla, western lowland gorilla, whale shark, whippet, white faced capuchin, white rhinoceros, white tiger, wild boar, wildebeest, wolf, wolverine, wombat, woodlouse, woodpecker, woolly mammoth, woolly monkey, wrasse, x-ray tetra, yak, yellow-eyed penguin, yorkshire terrier, zebra, zebra shark, zebu, zonkey, zorse, Turtles, Mammals]
//
////		acceptedAnimals.add("Turtles");
////		acceptedAnimals.add("Mammals");
//		acceptedAnimals.add("Human");
//		System.out.println(acceptedAnimals);
//		
//		OntologyCreator oCreator = new OntologyCreator();
//		OntModel animalModel = oCreator.getModel();
//	
//		//ObjectProperty eats = animalModel.createTransitiveProperty(baseURI + "eats");
//		ObjectProperty eats = animalModel.createObjectProperty(baseURI + "eats");
//		ObjectProperty eatenBy = animalModel.createObjectProperty(baseURI + "eatenBy");
//		eats.addInverseOf(eatenBy);
//
//		//while(animalsSoFar.size() <= 5){
//
//			try{
//				Testing.printTest(startAnimal, true);
//				//animalRecurse(startAnimal, acceptedAnimals,eats,oCreator,recurseChecker);
//				
//				animalsAddedSoFar = new LinkedList<String>();
//				
//				animalRecurse(startAnimal,acceptedAnimals,eats, eatenBy, oCreator,animalsAddedSoFar);
//			}
//
//			catch(Exception ex){
//				
//				//Change
//				System.out.println("Error");
//			}
//
////			OntClass Lion = oCreator.createOntClass("Lion");
////			OntClass Mouse = oCreator.createOntClass("Mouse");
////			OntClass Fox = oCreator.createOntClass("Fox");
////
////			Lion.addProperty(eats, Fox);
////			Fox.addProperty(eats, Mouse);
//			
//			animalModel = oCreator.getModel();
//			
//			OntologyCreator.printModel(animalModel);
//		
//	}


