package extractionTools;

import org.apache.log4j.BasicConfigurator;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import com.hp.hpl.jena.rdf.model.InfModel;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

//Class to extract information from the internet.
public class InformationExtractor {

	//Main loaded on execution.
	public static void main(String[] args) throws IOException{
		
		//Print that the class main has began execution.
		Testing.printTest("Start main",true);

		//Configure the basic logger for logging capabilities.
		BasicConfigurator.configure();

		//Create a list of animals
		//Example list could be of the form [Lion,Ostrich,Eagle,etc...]	
//		List<String> animalsToBeExtracted = animalListCreator(); 
	
		List<String> animalsToBeExtracted = new ArrayList<String>();
		
		//Extract the animals based on a certain URL.
//		List<String> possibleAnimals = animalListFinder("http://a-z-animals.com/animals/");
//		
////		System.out.println("A-Z list:" + possibleAnimals);
//		int i = 0;
//		for(String animal : possibleAnimals){
////			
////			//If multiple of 200 -> ensures only so many animals are being added.
////			//Limits the number of animals being tested,but also facilities to change this function.
//			if (i % 50 == 0){
//				
//				String[] spacedWord = animal.split(" ");
//				
//				//If the word does have spaces
//				if (spacedWord.toString() != ""){
//					
//					if (Arrays.asList(animal.split(" ")).contains(animal) == false){
//						//Use the join method to add in the underscore
//						animal = String.join("_", animal.split(" "));
//						animalsToBeExtracted.add(animal);
//					}
//				}
//				
//				//No spaces are present.
//				else{
//					animalsToBeExtracted.add(animal);
//				}
//			}
//			
//			i++;
//			
//		}
		
		//Add additional animals.
////	animalsToBeExtracted.add("LIon");
//		animalsToBeExtracted.add("lion");
//		animalsToBeExtracted.add("crocodile");
//		animalsToBeExtracted.add("bat");
//		animalsToBeExtracted.add("zebra_finch");
		
		String testanimal = "butterfly";
		
		animalsToBeExtracted.add(testanimal);
		
		Testing.printTest("Animal List chosen:" + animalsToBeExtracted,true);

		Testing.printTest("Passing list of animals & getting properties for each animal:",true);
		
//		//Get the extracted classes for each animal.
//		//e.g [Lion:Mammal,Mammalia]
		AnimalCounter animalClasses = AnimalClassExtractor.animalClassExtractor(animalsToBeExtracted);
		
		//The merge operation counts the number of times a certain classification has returned.
		//Outputs the most occuring classification or the last classified result(in the case of equal occurence).
		Map<String,String> animalClassDict= animalClasses.mergeResults();
	
		//Adds in additional properties such as animal Class properties as well as test data.
		//e.g Reptile:Scaled and Bird:wings 
		Map<String,String> animalPropertiesDict = AnimalCounter.addAdditionalPropertiesToDict(animalClassDict);
		System.out.println("Final animal dictionary of properties: " + animalPropertiesDict);

		//Create ontology which contains animals/animal classifications with the specified properties.
		Testing.printTest("Create ontology of animals and their properties:",true);
		
		OntologyCreator ontologyCreator = new OntologyCreator();
		//Using the pellet reasoner create an InferredModel which can be returned and printed.
		InfModel infAnimalModel = ontologyCreator.addProperties(animalPropertiesDict);
		//Print out ontology contents.
		OntologyCreator.printModel(infAnimalModel);
	
		//Food chain begins adding in additional relationships to the model.
		System.out.println("Adding food chain" + "\n" + "\n");
		InfModel finalAnimalModel= FoodChain.createFoodChain(testanimal, ontologyCreator);
		
		OntologyCreator.printModel(finalAnimalModel);
	}

	//Function to create a list of animals and return them.
	public static List<String> animalListCreator(){

		///String[] animalList = new String[] {"Goat","Sheep","Duck","Mushroom","Dragon","Shark",
		//"Ostrich","Goldfish","Tree","goldfis","whale","fish","frog"};

		///String[] //animalList = new String[] {"swordfish","Dolphin","Sloth","Possom","zebra","zebra_finch",
		///		"cerberus","chinchilla","marmoset","Chipmunk","eagle","lizard","Snake","mammoth","t-rex"};

		//String[] animalList = new String[] {"lion","vole","butterfly","human","snake","frog","Barracuda","crocodile","dodo","t-rex"};

		//List<String> animals = new ArrayList<String>(Arrays.asList("lion","vole","butterfly","human","snake","frog","Barracuda","crocodile","dodo"));

		//List<String> animals= new ArrayList<String>(Arrays.asList(animalList));
		
		List<String> animals= new ArrayList<String>();
		
		//animals.add("lion");
		animals.add("ostrich");
		//animals.add("Crocodile");
//					animals.add("crocodile");
//					animals.add("snapping-turtle");
//					animals.add("clownfish");
//					animals.add("Shark");
//					animals.add("Frog");
		//			
		//			
		////			animals.add("fox"); animals.add("butterfly");
		////			animals.add("bald-eagle"); animals.add("cow");
		////			animals.add("flying_fox");
		//						animals.add("owl");animals.add("clownfish");animals.add("snake");animals.add("eagle");

		return animals;

	}	
	
	//Provides a list of animals extracted from a given URL which has an "A-Z syntax.
	//Currently this function only supports "http://a-z-animals.com/animals/" but future extensions are possible.
	public static List<String> animalListFinder(String URL){
		
		List<String> animals = new ArrayList<String>();
		Document animalDocList = InformationExtractor.getHTML(URL);
		
		Elements em = animalDocList.select("table[class=article_az] ul li a");
		
		for (Element animal : em){
				animals.add(animal.attr("title").toLowerCase().trim());
		}
	
		return animals;
	}
	
	//Get the HTML for a given String and return this as a document.
	public static Document getHTML(String URL){

		String URI = "http://www.example.com/seanjames/document";
		//Set the returned string to empty string.

		Document doc = new Document(URI);

		try {

			// need http protocol //Create document with the extracted raw html.
			doc = Jsoup.connect(URL).get();

		} catch (IOException e) {
			System.out.println("JSoup URL Error: Page could not be accessed : " + URL);
		}

		return doc;
	}
	
	//Extract a set of Elements from a Document in the form of HTML.
	//This is achieved with a specified pattern to output a result.
	public static Elements patternExtractor(Document extractedHTML,String pattern){
		return extractedHTML.select(pattern);
	}

}
