//Imports 
package extractionTools;

import java.io.FileWriter;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.mindswap.pellet.jena.ModelExtractor;
import org.mindswap.pellet.jena.PelletReasonerFactory;

import com.hp.hpl.jena.ontology.ObjectProperty;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.Ontology;
import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;

//Class to create an ontology of animal property relationships and assert classifications.
public class OntologyCreator {

	//Create a global model from the factory as well as setting the ontology URI resource values.
	private static OntModel animalModel  = ModelFactory.createOntologyModel(PelletReasonerFactory.THE_SPEC);
	private final String baseURI = "http://www.example.com/seanjames/ontologies/animals/";
	private String ns = new String("http://www.example.com/seanjames/ontologies/animals#");

	//Create an ontology model and an ontology based on the top level URL.
	Ontology ontology = animalModel.createOntology(baseURI);
	OntClass ANIMAL = createOntClass("Animal");
	OntClass ANIMALCLASS = createOntClass("AnimalClass");

	//Simple get to return the model because it is stored globally.
	public OntModel getModel(){
		return animalModel;
	}
	//Add properties associated with each animal and return and inferred model of assertions.	
	public InfModel addProperties(Map<String,String> animalRelations){

		System.out.println(animalRelations);
		
		//For each animal and property pair. {[Ostrich,Bird#wings],...} 
		for(Map.Entry<String, String> animalPair : animalRelations.entrySet()){

				String animalKey = "";
				String probableClassification = "NONEFOUND";
				
				System.out.println("\n");
				if(animalPair.getKey().contains("#")){
					
					String[] splitKey= animalPair.getKey().split("#");
					animalKey = splitKey[0];
					probableClassification = splitKey[1];
				}
				
				else{
					
					animalKey = animalPair.getKey();
				}
				
				List<String> animalClassesArray = Arrays.asList("Mammal","Reptile","Amphibian","Fish","Bird","Unclassified");
				
				Set<String> animalClasses = new HashSet<String>(animalClassesArray);
				 
				//Create an array of the properties to be added.
				String[] properties = animalPair.getValue().split(",");
				
				System.out.println("Properties for:" + animalKey + ":" + Arrays.toString(properties) + "\nShould be classifed as:" + probableClassification);
				
//				//Incase the array contains more than one delimited field in the future.
//				//The last propertyString is the one which dominates.
////				for(String propertiesString : propertiesArray){
////						properties = propertiesString.split(",");
////						
////				}
//				
//				if(properties[0] == "NOPROPERTIESAVAILABLE"){
//					
//				}
				
				
				//If there are properties to be added.
				if (properties.length >= 1){
					
						//Change to dictionary form
						if(animalClasses.contains(animalKey) || probableClassification == "ANIMALCLASS"){
							System.out.println("Class being added:" + animalKey);

							OntClass animalOntClass = createOntClass(animalKey);

							for (int i = 0 ; i < properties.length ; i++){
								System.out.println("Property being added:" + properties[i]);

								//Create a new property to be added to the animal.
								ObjectProperty p = createProperty(properties[i], ANIMAL, ANIMAL);

								//Add restricted [null,property,some animal] relationship between the animal.
								createAdvancedRelationship(animalOntClass, "equivalent", null, p, ANIMAL);
								animalOntClass.addSuperClass(ANIMAL);

							}
						}

						else{
							System.out.println("Animal being added:" + animalKey);
							//Create the appropriate OntClasses.
							OntClass animal = createOntClass(animalKey);

							//Make the animalClassification a subclass of the top level "ANIMAL" OntClass.
							//ANIMAL.addSubClass(animalClass);

							for (int i = 0 ; i < properties.length ; i++){
								System.out.println("Property being added:" + properties[i]);

								if((properties[i].split("%").toString() != "") && (properties[i].split("%").length > 1)){
									String[] splitScientificWords = properties[i].split("%");
									System.out.println("Split result" + Arrays.toString(splitScientificWords));
									
									for(String word : splitScientificWords){
										
										ObjectProperty scientificName = createProperty("#hasScientificName", ANIMAL, ANIMAL);

										OntClass scientificClass = createOntClass(word);
										
										if(!word.equals("latinName")){
											animal.addProperty(scientificName, scientificClass);
										}
									}
									
								}
								
								else{
								//Create a new property to be added to the animal.
								ObjectProperty p = createProperty(properties[i], ANIMAL, ANIMAL);

								//Add restricted [null,property,some animal] relationship between the animal.
								createAdvancedRelationship(animal, "restricted", null, p, ANIMAL);
								animal.addSuperClass(ANIMAL);
								}

							}

						}
						
			}

		}

		System.out.println("\n");
		animalModel.write(System.out,"Turtle");

		return ModelFactory.createInfModel( PelletReasonerFactory.theInstance().create(), animalModel);

	}

	//Create the OntClass object based on a given String and the ns value.
	public OntClass createOntClass(String aclass){
		return animalModel.createClass(ns + aclass);
	}

	//Function to add a more advanced relationship to the ontology.
	public void createAdvancedRelationship(OntClass animal, String type, String v1, ObjectProperty p, OntClass someThing){
		//If the type is set to restricted it will create a "someValuesFromRestriction" relation.(add relation to create assertion)
		if(type.equalsIgnoreCase("restricted")){
			animal.addSuperClass( animalModel.createSomeValuesFromRestriction( null, p, someThing ));
		}
		
		//Else it will create a "equivalentClass" relation.(mainly for animalClassification assertions)
		else if (type.equalsIgnoreCase("equivalent")){
			animal.addEquivalentClass(animalModel.createSomeValuesFromRestriction( null, p, someThing ));
		}

	}

	//Function to create and return an ObjectProperty with the domain and range also added.
	public ObjectProperty createProperty(String propertyName,OntClass domain,OntClass range){
		ObjectProperty p = animalModel.createObjectProperty(baseURI + propertyName);
		p.addDomain(domain); p.addRange(range);
		return p;
	}
	
	//Function to create and return an ObjectProperty 
	public ObjectProperty createBasicProperty(String propertyName){
		ObjectProperty p = animalModel.createObjectProperty(baseURI + propertyName);
		return p;
	}

	//Print the model firstly without inference and then the inferred model.
	public static void printModel(Model animalModel){
		
		FileWriter fw = null;

		Testing.printTest("Starting model(without inference):", true);
		animalModel.write(System.out,"TURTLE");

		// Create an inference extractor to get required model info.
		ModelExtractor extractor = new ModelExtractor(animalModel);

		// Extract default set of inferences
		Model inferences = extractor.extractModel();

		Testing.printTest("Inferred model:", true);
		inferences.write(System.out,"TURTLE");
		
		try {
            fw = new FileWriter("inferences.owl");
            inferences.write( fw, "Turtle" );
            fw.close();
		}
		
		catch (Exception ex) {
			ex.printStackTrace();
        }
	}
	
	//Function to create and return an ObjectProperty 
	public void addTriple( OntClass predClass,ObjectProperty property,OntClass preyClass){
		
		//Add the subject,predicate object triple relation to the ontology.
		predClass.addProperty(property, preyClass);
		
	}

}
