package extractionTools;

import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

public class PropertyExtractor {

	//Function which finds all the properties present in a specific animal's set of URLs.
	public static String propertyFind(String animal) throws IOException{

		StringBuilder builder = new StringBuilder();
		String scientificName = "";
		Map<String,Boolean> properties = new HashMap<String,Boolean>();
		Set<String> addedSoFar = new HashSet<String>();
		@SuppressWarnings("unused")Boolean propertiesFound = false;
		Boolean scientificNameAdded = false;
		List<String> listLinks = Arrays.asList("http://animals.sandiegozoo.org/animals/","http://a-z-animals.com/animals/");

		//For each link in the list.
		for(String link : listLinks){

			//Extract all the HTML
			Document extractedHTML = InformationExtractor.getHTML(link + animal);

			//Populate a dictionary of whether or not the property is found.
			properties.put("scaled" + "#" + link, extractedHTML.select(":matches((?i)^scales$)").hasText()) ;
			properties.put("coldBlooded" + "#" + link, extractedHTML.select(":matches((?i)^cold?blooded$)").hasText()) ;
			properties.put("gills" + "#" + link, extractedHTML.select(":matches((?i)gills)").hasText());
			properties.put("vertebrate" + "#" + link, extractedHTML.select(":matches((?i)vertebrat*)").hasText());
			properties.put("winged" + "#" + link,extractedHTML.select(":matches((?i)feather*), :matches((?i)^wings$)").hasText());
			properties.put("hairy" + "#" + link,extractedHTML.select(":matches((?i)^fur$), :matches((?i)hair), :matches((?i)suckle)").hasText());

			//If the scientific name was not previously found.
			if (scientificNameAdded == false){

				//Try to extract the information.
				try{
					String pattern = "table[class = article_facts] tr:contains(Scientific Name:)";
					Elements interimClass = InformationExtractor.patternExtractor(extractedHTML,pattern);

					Element latinNameExtracted = (Element) interimClass.select("td").first().nextSibling();
					scientificName = latinNameExtracted.text();
					
					//As longs as the scientific name has a value add this to the properties to be added.
					if(scientificName != ""){
						System.out.println("Scientific name added:" + scientificName + " from source:" + extractedHTML.baseUri() );
						properties.put("hasScientificName%" + scientificName + "#" + link,true);
						scientificNameAdded = true;
					}

				}
				
				//Scientific name was not found.
				catch(Exception ex){
					System.out.println("Scientific name cannot be found:" + extractedHTML.baseUri());
				}
				
			}
		}
		
		System.out.println(properties);

		//Add each of the properties added by splitting them from the corresponding URL.
		for( Map.Entry<String, Boolean> propertyURLResult : properties.entrySet() ){
			//System.out.println(property.getKey() + ":" + property.getValue());

			String property = propertyURLResult.getKey().split("#")[0];

			//Longs as the value has not been added so far and the property was found.
			if(propertyURLResult.getValue() == true && !addedSoFar.contains(property)){
				
				propertiesFound = true;

				if(builder.length() != 0){
					builder.append(",");
				}

				//Add the property.
				builder.append(property);
				addedSoFar.add(property);
			}

		}

		//If some properties have been found on the page.
		if(builder.length() != 0){
			return builder.toString();
		}

		//Else : No properties have been added.
		//So return this fact.
		else if(propertiesFound = false){
			return builder.append("NOPROPERTIESFOUND").toString();
		}
		
		//Just in case.
		else{
			return builder.append("NOPROPERTIESFOUND").toString();
		}
	}


}